/******************************************************************************/
/*  (C) COPYRIGHT International Business Machines Corp. 1993                  */
/*                            All Rights Reserved                             */
/*                                                                            */
/*  INTERNATIONAL BUSINESS MACHINES CORPORATION PROVIDES THIS MATERIAL        */
/*  "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,        */
/*  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY  */
/*  AND FITNESS FOR A PARTICULAR PURPOSE.                                     */
/*                                                                            */
/******************************************************************************/

#include <X11/PEX5/PEXlib.h>
#include <X11/PEX5/PEXExtlib.h>
#include <misc.h>
#include <common_ibm.h>

Window local_win;

/****************************************************************************
 *  Tests circles                                                           *
 ***************************************************************************/
void draw_circle(Display          *dpy,
               XID              resourceID,
               PEXOCRequestType req_type)

  {
    PEXCoord2D  center; 
    double radius;
    PEXColor    int_color, edge_color;

    /*****************************************************************
     Top Row. Draws basic ellipses with default attributes
    *****************************************************************/
    
    radius = 0.14;
    center.x = 0.17;
    center.y = 0.80;

    PEXExtCircle2D(dpy, resourceID, req_type, &center, radius);
    center.x = 0.49;

    if (enum_type_supported(dpy, local_win, PEXETCurveApproxMethod,
                            PEXApproxDCChordalSize)) {
      PEXSetCurveApprox(dpy, resourceID, req_type, PEXApproxDCChordalSize,
        2.0);
    }
    else {
      printf("PEXApproxDCChordalSize curve approx not supported\n");
    }

    PEXExtCircle2D(dpy, resourceID, req_type, &center, radius);

    if (enum_type_supported(dpy, local_win, PEXETCurveApproxMethod,
                            PEXApproxConstantBetweenKnots)) {
      PEXSetCurveApprox(dpy, resourceID, req_type, 
                        PEXApproxConstantBetweenKnots, 6.0);
    }
    else {
      printf("PEXApproxConstantBetweenKnots curve approx not supported\n");
    }

    center.x = 0.81;
    PEXExtCircle2D(dpy, resourceID, req_type, &center, radius);

    /*****************************************************************
     Middle Row.                                                 
    *****************************************************************/
    int_color.rgb.red = 1.0;
    int_color.rgb.green = 1.0;
    int_color.rgb.blue = 0.0;
    PEXSetSurfaceColor(dpy, resourceID, req_type, PEXColorTypeRGB,
      &int_color);

    PEXSetCurveApprox(dpy, resourceID, req_type, PEXApproxImpDep, 1.0);
    PEXSetInteriorStyle(dpy, resourceID, req_type, PEXInteriorStyleSolid);
    PEXSetSurfaceEdgeFlag(dpy, resourceID, req_type, PEXOn);
    center.x = 0.5;
    center.y = 0.5;
    PEXExtCircle2D(dpy, resourceID, req_type, &center, radius);

    /*****************************************************************
     Bottom Row.                                                 
    *****************************************************************/
    int_color.indexed.index = 4;
    PEXSetSurfaceColor(dpy, resourceID, req_type, PEXColorTypeIndexed,
      &int_color);
    PEXSetSurfaceEdgeType(dpy, resourceID, req_type, PEXSurfaceEdgeDashed);
    center.x = 0.25;
    center.y = 0.2;
    PEXExtCircle2D(dpy, resourceID, req_type, &center, radius);

    edge_color.rgb.red = 1.0;
    edge_color.rgb.green = 0.0;
    edge_color.rgb.blue = 0.0;
    PEXSetSurfaceEdgeColor(dpy, resourceID, req_type,PEXColorTypeRGB, 
      &edge_color);
    PEXSetInteriorStyle(dpy, resourceID, req_type, PEXInteriorStyleHatch);

    center.x = 0.75;
    PEXExtCircle2D(dpy, resourceID, req_type, &center, radius);
  }

/****************************************************************************
 *  Sets the test parameters                                                *
 ***************************************************************************/
void inquire_test_params(char *test_name,
                         int  *num_images,
                         int  *supported_modes,
                         char *win_title)


  {
    strcpy(test_name, "circle");
    *num_images = 1;
    *supported_modes = IMM_MODE | STR_MODE;

    strcpy(win_title, "Circle");
  }


/****************************************************************************
 *  This routine is used to do miscellaneous things like setting some       *
 *  renderer attributess, Viewing etc if needed.                            *
 ***************************************************************************/
void misc_setup(Display *dpy, Window window, PEXRenderer renderer,
                int cur_image)
  {
      local_win = window;
  }

/****************************************************************************
 *  This routine calls the function which does the actual test              *
 ***************************************************************************/
void execute_test(Display *dpy, XID resourceID,
               PEXOCRequestType req_type, int cur_image)
  {
    describe_test( "\n" );
    describe_test( " Tests Circle with different Attributes\n" );
    describe_test( "\n" );
    describe_test( " TOP ROW: 3 circles. First one is renedered with default attributes.\n");
    describe_test( "          Second is with curve approximation of PEXApproxDCChordalSize.\n");
    describe_test( "          Third with curve approximation of PEXApproxConstantBetweenKnots.\n");
    describe_test( " MIDDLE ROW: Interior style solid and interior color Yellow, Edge ON \n");
    describe_test( " THIRD ROW: Two circles. Edge type Dashed. The first one has green interior\n");
    describe_test( "            and the second one has red edge and interior is hatched.\n"); 
    describe_test( "\n"); 

    draw_circle(dpy, resourceID, req_type);

  }

void testproc(Display *dpy, Window window, int cur_image)
  {
    image_testproc("circle", "circle test", img_desc,
                    dpy,window, 0, 0,
                    glob_window_width, glob_window_height);
  }

