/******************************************************************************/
/*                                                                            */
/* $Source: /src/rcs/Cats/XSystem/pex/mbx/common.c,v $
/* $Date: 93/12/30 13:52:55 $
/* $Revision: 1.3 $
/*                                                                            */
/* Description:                                                               */
/*	Common utilities for MBX test programs				      */
/******************************************************************************/


/******************************************************************************/
/*                                                                            */
/* Include Files                                                              */
/*                                                                            */
/******************************************************************************/

/* ------ system and platform files ------------------------------------------*/
#include <X11/PEX5/PEXlib.h>
#include <X11/PEX5/PEXHPlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/multibuf.h>
#include <stdio.h>
#include <math.h>

/* ------ program specific ---------------------------------------------------*/
#include "pexutcmap.h"
#include "common.h"

/* ------ global declarations ------------------------------------------------*/

static PEXCoord	Top[]    = {{.4,.6,.4}, {.4,.6,.6},  /* Top face of cube      */
			    {.6,.6,.6}, {.6,.6,.4},
			    {.4,.6,.4}};

static PEXCoord	Bottom[] = {{.4,.4,.4}, {.4,.4,.6},  /* Bottom face of cube   */
			    {.6,.4,.6}, {.6,.4,.4},
			    {.4,.4,.4}};

static PEXCoord	Front[]  = {{.4,.6,.6}, {.4,.4,.6},  /* Front face of cube    */
			    {.6,.4,.6}, {.6,.6,.6},
			    {.4,.6,.6}};

static PEXCoord	Back[]   = {{.4,.6,.4}, {.4,.4,.4},  /* Back face of cube     */
			    {.6,.4,.4}, {.6,.6,.4},
			    {.4,.6,.4}};

static PEXCoord Left[]   = {{.4,.6,.4}, {.4,.6,.6},  /* Left face of cube     */
			    {.4,.4,.6}, {.4,.4,.4},
			    {.4,.6,.4}};

static PEXCoord Right[]  = {{.6,.6,.6}, {.6,.4,.6},  /* Right face of cube    */
                            {.6,.4,.4}, {.6,.6,.4},
			    {.6,.6,.6}};

/******************************************************************************/
/*                                                                            */
/* Function Definitions                                                       */
/*                                                                            */
/******************************************************************************/

/******************************************************************************/
Display *
init_pex(argv, pexinfo)
    char                *argv[];
    PEXExtensionInfo    **pexinfo;

/*  Purpose: This function initializes PEX and reports any errors.
 *                  
 *  Return:  Display pointer
 *
 ******************************************************************************/
{
    char        *dpy_name, err[80];
    Display     *dpy = (Display *)NULL;

    if ( argv[1] && strcmp( "-display", argv[1] ) == 0 )
        dpy_name = XDisplayName( argv[2] );
    else
        dpy_name = XDisplayName( NULL );

    if ( !(dpy = XOpenDisplay( dpy_name )) ){
        fprintf ( stderr, "Could not open display %s\n", dpy_name );

    } else if ( PEXInitialize( dpy, pexinfo, 80, err ) != 0 ) {
        fprintf( stderr, "No PEX extension on %s.\n", dpy_name );
        fprintf( stderr, "%s\n", err );
        XCloseDisplay( dpy );
        dpy = (Display *)NULL;
    }

    return dpy;
}


/******************************************************************************/
PEXRenderer
My_setup_renderer( dpy, window, capx_info, attrs )
    Display                     *dpy;
    Window                      window;
    PEXColorApproxEntry         *capx_info;
    PEXRendererAttributes       *attrs;

/*  Purpose: This function sets up a renderer with necessart LUTS.
 *                  
 *  Return:  Renderer ID
 *
 ******************************************************************************/
{
    unsigned long               mask = 0;
    PEXRenderer                 renderer;

    mask |= PEXRAInteriorBundle;
    attrs->interior_bundle =
        PEXCreateLookupTable( dpy, window, PEXLUTInteriorBundle );

    mask |= PEXRAEdgeBundle;
    attrs->edge_bundle =
        PEXCreateLookupTable( dpy, window, PEXLUTEdgeBundle );

    /* Create a color table. */
    mask |= PEXRAColorTable;
    attrs->color_table =
        PEXCreateLookupTable( dpy, window, PEXLUTColor );

    /* Create a view table. */
    mask |= PEXRAViewTable;
    attrs->view_table =
        PEXCreateLookupTable( dpy, window, PEXLUTView );

    /* Create a light table. */
    mask |= PEXRALightTable;
    attrs->light_table =
        PEXCreateLookupTable( dpy, window, PEXLUTLight );

    /* Create a depth cue table. */
    mask |= PEXRADepthCueTable;
    attrs->depth_cue_table =
        PEXCreateLookupTable( dpy, window, PEXLUTDepthCue );

    /* Create a color approximation table and set the default */
    /* entry, entry 0, to the colormap approximation specified. */
    mask |= PEXRAColorApproxTable;    attrs->color_approx_table =
        PEXCreateLookupTable( dpy, window, PEXLUTColorApprox );
    PEXSetTableEntries( dpy, attrs->color_approx_table, 0, 1,
                        PEXLUTColorApprox, capx_info );

    /* Create the renderer */
    renderer = PEXCreateRenderer( dpy, window, mask, attrs );

    return renderer;
}


/******************************************************************************/
void
redraw( Dpy, Win, Renderer, Angle, ColorFlag, CubeType )
	Display    *Dpy;        /* The display for the cube                   */
	Window      Win;        /* The window for the cube                    */
	PEXRenderer Renderer;   /* The renderer for the cube                  */
	double      Angle;      /* The angle of rotation                      */
	int         ColorFlag;  /* The current color of the cube              */
	int         CubeType;   /* The current style of the cube              */

/*  Purpose: This function will be called when the cube needs to be drawn     */
/*           at a new angle.  redraw will create all of the attribute matrices*/
/*           and will change the color of the cube if necessary, and will     */
/*           change the cube type if necessary                                */
/*                                                                            */
/*  Return:  NOTHING                                                          */
/*                                                                            */
/******************************************************************************/
{
/* ------ redraw data --------------------------------------------------------*/
    PEXVector		Shift;                   /* Position cube for rotation*/
    PEXMatrix    	TranslateMatrix,         /* Matrix values for trans.  */
			RotateMatrixX,           /* Rotate in X dimension     */
			RotateMatrixY,           /* Rotate in Y dimension     */
			RotateMatrixZ,           /* Rotate in Z dimension     */
			IntMatrix,               /* Intermediate matrix       */
			IntMatrix1,              /* Intermediate matrix       */
			IntMatrix2,              /* Intermediate matrix       */
                        ResultMatrix;            /* Final Matrix              */

/* ------ redraw code --------------------------------------------------------*/

    PEXBeginRendering( Dpy, Win, Renderer );
    {
	/*
	** Translate center of bar to the origin.
	*/
	Shift.x = Shift.y = Shift.z =  -0.5;
	PEXTranslate( &Shift, TranslateMatrix );
	PEXSetLocalTransform( Dpy, Renderer, PEXOCRender, PEXReplace,
			      TranslateMatrix);

	/*
	** Apply a rotation of "Angle" radians to the Z Axis
	*/
	PEXRotate(PEXZAxis, Angle, RotateMatrixZ);

	/*
	** Apply a rotation of "Angle" radians to the X Axis
	*/
	PEXRotate(PEXXAxis, Angle, RotateMatrixX);
	PEXMatrixMult (RotateMatrixZ, RotateMatrixX, IntMatrix1);

	/*
	** Apply a rotation of "Angle" radians to the Y Axis
	*/
	PEXRotate(PEXYAxis, Angle, RotateMatrixY);
	PEXMatrixMult (IntMatrix1, RotateMatrixY, IntMatrix2);

	PEXSetLocalTransform( Dpy, Renderer, PEXOCRender, PEXPostConcatenate,
			      IntMatrix2);

	/*
	** Translate Back to center of screen.
	*/
	Shift.x = Shift.y = Shift.z = 0.5;
	PEXTranslate( &Shift, TranslateMatrix );

	PEXSetLocalTransform( Dpy, Renderer, PEXOCRender, PEXPostConcatenate,
			      TranslateMatrix);

        /*
	** Set up the cube style:
	** WIREFRAME: Render the cube with only the edges showing in the
	**            current value of ColorFlag.
	** SOLID: Render the cube with both edges and faces showing.  Edges
	**        will be in black, and the faces will be determined by the
	**        ColorFlag value
	*/
        switch (CubeType)
	{
	  case WIREFRAME : PEXSetSurfaceEdgeFlag ( Dpy, Renderer, PEXOCRender,
				                   PEXSurfaceEdgeSolid);

                           PEXSetInteriorStyle( Dpy, Renderer, PEXOCRender,
		                                PEXInteriorStyleEmpty );

	                   PEXSetSurfaceEdgeColorIndex ( Dpy, Renderer,
							 PEXOCRender,ColorFlag);
                           break;

          case SOLID     : PEXSetSurfaceEdgeFlag ( Dpy, Renderer, PEXOCRender,
                	                           PEXSurfaceEdgeSolid);

	                   PEXSetInteriorStyle( Dpy, Renderer, PEXOCRender,
			                        PEXInteriorStyleSolid );

                           PEXSetSurfaceColorIndex (Dpy, Renderer, PEXOCRender,
						    ColorFlag);

	                   PEXSetSurfaceEdgeColorIndex (Dpy, Renderer,
							PEXOCRender,BLACKLINES);
                           break;

          default        : break;
        }

        /*
	** Render all faces of the cube:
	** WIREFRAME: Render only the FRONT, BACK, TOP, and BOTTOM sides
	** SOLID: Render FRONT, BACK, TOP, BOTTOM, LEFT and RIGHT sides.
	**        If LEFT and RIGHT sides are not Filled, the cube will
	**        appear to only have 4 sides.
	*/
        PEXFillArea( Dpy, Renderer,PEXOCRender,PEXShapeConvex, False, 5,Front);
        PEXFillArea( Dpy, Renderer,PEXOCRender,PEXShapeConvex, False, 5,Back);
        PEXFillArea( Dpy, Renderer,PEXOCRender,PEXShapeConvex, False, 5,Top);
        PEXFillArea( Dpy, Renderer,PEXOCRender,PEXShapeConvex, False, 5,Bottom);

        if (CubeType == SOLID)
	{
            PEXFillArea( Dpy, Renderer, PEXOCRender, PEXShapeConvex, False, 5,
			 Left );
            PEXFillArea( Dpy, Renderer, PEXOCRender, PEXShapeConvex, False, 5,
			 Right );
        }


    }
    PEXEndRendering( Dpy, Renderer, True );
    XFlush( Dpy );
}

/******************************************************************************/
void
SetView( Dpy, view_table )
	Display *Dpy;
	PEXLookupTable view_table;

/*  Purpose: This function will set the view orientation parameters for the   */
/*           display.                                                         */
/*                                                                            */
/*  Return:  NOTHING                                                          */
/*                                                                            */
/******************************************************************************/
{
/* ------ SetView data -------------------------------------------------------*/
    PEXViewEntry	view;
    PEXCoord2D		Win[2];
    double		view_plane,
			Front_plane,
			Back_plane;
    PEXCoord		prp;
    PEXNPCSubVolume	viewport;
    static PEXCoord	view_ref_pt = {0.5, 0.5, 0.5};
    static PEXVector	view_plane_normal = {2, 0.5, 0.5};
    static PEXVector	view_up_vec = {0,1,0};


/* ------ SetView code -------------------------------------------------------*/
    /*
    ** Compute the view orientation transform.
    */
    PEXViewOrientationMatrix( &view_ref_pt, &view_plane_normal,
			      &view_up_vec, view.orientation );

    /*
    ** The view mapping parameters.
    */
    prp.x = 0;
    prp.y = 0;
    prp.z = 20;
    Win[0].x = -0.5;
    Win[1].x = 0.5;
    Win[0].y = -0.5;
    Win[1].y = 0.5;
    Front_plane = 100;
    view_plane = 0;
    Back_plane = -100;
    viewport.min.x = 0;
    viewport.max.x = 1;
    viewport.min.y = 0;
    viewport.max.y = 1;
    viewport.min.z = 0;
    viewport.max.z = 1;

    /*
    ** Compute the view mapping transform.
    */
    PEXViewMappingMatrix( Win, &viewport, False, &prp, view_plane, Back_plane,
			  Front_plane, view.mapping );

    /*
    ** The view clipping parameters.
    */
    view.clip_flags = PEXClippingAll;
    view.clip_limits = viewport;

    /*
    ** Set view 1.
    */
    PEXSetTableEntries( Dpy, view_table, 1, 1, PEXLUTView,(PEXPointer)&view );
}

/******************************************************************************/
void
SetColors ( Dpy, color_table )
	Display *Dpy;                    /* Display for the color table       */
	PEXLookupTable color_table;      /* The color table we will remap     */

/*  Purpose: This function will set remap NUMCOLORS locations in the colormap */
/*           starting at location REDLINES.                                   */
/*                                                                            */
/*  Return:  NOTHING                                                          */
/*                                                                            */
/******************************************************************************/
{
/* ------ SetColor data ------------------------------------------------------*/

  PEXColorSpecifier colors[NUMCOLORS]; /* color values to insert into colormap*/


/* ------ SetColor code ------------------------------------------------------*/
  /*
  ** Define the color: RED
  */
  colors[0].type = PEXColorTypeRGB;
  colors[0].value.rgb.red = 1.0;
  colors[0].value.rgb.green = 0.2;
  colors[0].value.rgb.blue = 0.2;

  /*
  ** Define the color: GREEN
  */
  colors[1].type = PEXColorTypeRGB;
  colors[1].value.rgb.red = 0.2;
  colors[1].value.rgb.green = 1.0;
  colors[1].value.rgb.blue = 0.2;

  /*
  ** Define the color: BLUE
  */
  colors[2].type = PEXColorTypeRGB;
  colors[2].value.rgb.red = 0.2;
  colors[2].value.rgb.green = 0.2;
  colors[2].value.rgb.blue = 1.0;

  /*
  ** Define the color: BLACK
  */
  colors[3].type = PEXColorTypeRGB;
  colors[3].value.rgb.red = 0.0;
  colors[3].value.rgb.green = 0.0;
  colors[3].value.rgb.blue = 0.0;

  /*
  ** Update the color lookup table
  */
  PEXSetTableEntries (Dpy, color_table, REDLINES, NUMCOLORS, PEXLUTColor,
		      (PEXPointer) colors);
}
